@@ -3,32 +3,33 @@ package Pcore::API::Client;
 use Pcore -class, -res;
 
 # use Pcore::WebSocket;
-use Pcore::Lib::Scalar qw[is_callback is_plain_arrayref is_plain_coderef weaken];
-use Pcore::Lib::Data qw[to_cbor from_cbor];
+use Pcore::Lib::Scalar qw[is_plain_arrayref];
+use Pcore::Lib::Data qw[to_cbor from_cbor to_json from_json];
 use Pcore::Lib::UUID qw[uuid_v1mc_str];
 use Pcore::HTTP qw[:TLS_CTX];
 
 has uri => ( required => 1 );    # InstanceOf ['Pcore::Lib::URI'], http://token@host:port/api/, ws://token@host:port/api/
 
-has token   => ();
-has api_ver => ();               # eg: 'v1', default API version for relative methods
+has token    => ();
+has api_ver  => ();              # eg: 'v1', default API version for relative methods
+has use_json => ();
 
-has tls_ctx         => $TLS_CTX_HIGH;    # Maybe [ HashRef | Enum [ $TLS_CTX_LOW, $TLS_CTX_HIGH ] ]
+has timeout         => ();
 has connect_timeout => 10;
-
-# HTTP options
-has persistent => 600;
-has timeout    => ();
+has tls_ctx         => $TLS_CTX_HIGH;    # Maybe [ HashRef | Enum [ $TLS_CTX_LOW, $TLS_CTX_HIGH ] ]
+has bind_ip         => ();
 
 # WebSocket options
-has compression    => 0;
-has bind_events    => ();
-has forward_events => ();
-has on_connect     => ();                # Maybe [CodeRef]
-has on_disconnect  => ();                # Maybe [CodeRef]
-has on_rpc         => ();                # Maybe [CodeRef]
-has on_ping        => ();                # Maybe [CodeRef]
-has on_pong        => ();                # Maybe [CodeRef]
+has bindings         => ();
+has max_message_size => 1_024 * 1_024 * 100;
+has compression      => 0;
+has on_connect       => ();                    # Maybe [CodeRef]
+has on_disconnect    => ();                    # Maybe [CodeRef]
+has on_bind          => ();                    # Maybe [CodeRef]
+has on_event         => ();                    # Maybe [CodeRef]
+has on_rpc           => ();                    # Maybe [CodeRef]
+has on_ping          => ();                    # Maybe [CodeRef]
+has on_pong          => ();                    # Maybe [CodeRef]
 
 has _is_http => ( required => 1 );
 
@@ -58,7 +59,9 @@ sub set_token ( $self, $token = undef ) {
     if ( $token // $EMPTY ne $self->{token} // $EMPTY ) {
         $self->{token} = $token;
 
-        $self->disconnect;
+        my $h = $self->{_ws};
+
+        $h->auth($token) if defined $h;
     }
 
     return;
@@ -101,16 +104,14 @@ sub _send_http ( $self, $method, $args ) {
     my $res = P->http->post(
         $self->{uri},
         connect_timeout => $self->{connect_timeout},
-
-        # persistent      => $self->{persistent},
-        tls_ctx => $self->{tls_ctx},
+        tls_ctx         => $self->{tls_ctx},
         ( $self->{timeout} ? ( timeout => $self->{timeout} ) : () ),
         headers => [
             Referer        => undef,
             Authorization  => "Token $self->{token}",
-            'Content-Type' => 'application/cbor',
+            'Content-Type' => $self->{use_json} ? 'application/json' : 'application/cbor',
         ],
-        data => to_cbor($payload)
+        data => $self->{use_json} ? to_json $payload : to_cbor $payload
     );
 
     if ( !$res ) {
@@ -136,11 +137,9 @@ sub _send_http ( $self, $method, $args ) {
 }
 
 sub _send_ws ( $self, $method, $args ) {
-    my ( $ws, $error ) = $self->_get_ws;
+    my $ws = $self->_get_ws;
 
-    return $error if defined $error;
-
-    return $ws->rpc_call( $method, $args );
+    return $ws->rpc_call( $method, $args->@* );
 }
 
 sub _get_ws ( $self ) {
@@ -156,87 +155,38 @@ sub _get_ws ( $self ) {
 
     $self->{_get_ws_threads} = 1;
 
-    weaken $self;
-
-    my $cv = P->cv;
-
-    Pcore::WebSocket->connect_ws(
+    my $h = Pcore::WebSocket::pcore->connect(
         $self->{uri},
-        protocol         => 'pcore',
-        max_message_size => 0,
+
+        # connection
+        timeout         => $self->{timeout} // 30,
+        connect_timeout => $self->{connect_timeout},
+        tls_ctx         => $self->{tls_ctx},
+        bind_ip         => $self->{bind_ip},
+
+        # websocket
+        max_message_size => $self->{max_message_size},
         compression      => $self->{compression},
-        connect_timeout  => $self->{connect_timeout},
-        tls_ctx          => $self->{tls_ctx},
-        before_connect   => {
-            token          => $self->{token},
-            bind_events    => $self->{bind_events},
-            forward_events => $self->{forward_events},
-        },
-        on_listen_event => sub ( $ws, $mask ) {    # API server can listen client events
-            return 1;
-        },
-        on_fire_event => sub ( $ws, $key ) {       # API server can fire client events
-            return 1;
-        },
-        on_connect_error => sub ($res) {
-            $cv->( undef, $res );
 
-            return;
-        },
-        on_connect => sub ( $ws, $headers ) {
-            $self->{_ws} = $ws;
-
-            $self->{on_connect}->( $self, $headers ) if defined $self->{on_connect};
-
-            $cv->( $ws, undef );
-
-            return;
-        },
-        on_disconnect => sub ( $ws, $status ) {
-            undef $self->{_ws};
-
-            $self->{on_disconnect}->( $self, $status ) if $self && $self->{on_disconnect};
-
-            return;
-        },
-        on_ping => do {
-            if ( $self->{on_ping} ) {
-                sub ( $ws, $payload_ref ) {
-                    $self->{on_ping}->( $self, $payload_ref ) if $self && $self->{on_ping};
-
-                    return;
-                };
-            }
-        },
-        on_pong => do {
-            if ( $self->{on_pong} ) {
-                sub ( $ws, $payload_ref ) {
-                    $self->{on_pong}->( $self, $payload_ref ) if $self && $self->{on_pong};
-
-                    return;
-                };
-            }
-        },
-        on_rpc => do {
-            if ( $self->{on_rpc} ) {
-                sub ( $ws, $req, $tx ) {
-                    $self->{on_rpc}->( $self, $req, $tx ) if $self && $self->{on_rpc};
-
-                    return;
-                };
-            }
-        },
+        # pcore websocket
+        use_json      => $self->{use_json},
+        token         => $self->{token},
+        bindings      => $self->{bindings},
+        on_disconnect => sub { delete $self->{_ws} },
+        on_bind       => $self->{on_bind},
+        on_event      => $self->{on_event},
+        on_rpc        => $self->{on_rpc},
     );
 
-    my ( $ws, $error ) = $cv->recv;
+    $self->{_ws} = $h if $h;
 
     $self->{_get_ws_threads} = 0;
 
     while ( my $cb = shift $self->{_get_ws_queue}->@* ) {
-        $cb->( $ws, $error );
+        $cb->($h);
     }
 
-    return $ws, $error;
+    return $h;
 }
 
 1;
