@@ -2,8 +2,7 @@ package Pcore::App::API::Auth;
 
 use Pcore -class, -res;
 use Pcore::App::API qw[:TOKEN_TYPE :PRIVATE_TOKEN :PERMISSIONS];
-use Pcore::App::API::Request;
-use Pcore::Lib::Scalar qw[is_callback is_plain_coderef is_plain_arrayref];
+use Pcore::Lib::Scalar qw[is_res is_plain_arrayref];
 
 use overload    #
   q[bool] => sub {
@@ -22,12 +21,7 @@ has permissions   => ();                      # Maybe [HashRef]
     return { $self->%{qw[is_authenticated user_id user_name permissions]} };
 };
 
-sub TO_DUMP ( $self, $dumper, @ ) {
-    my %args = (
-        path => undef,
-        splice @_, 2,
-    );
-
+sub TO_DUMP ( $self, $dumper, %args ) {
     my $tags;
 
     my $res;
@@ -85,78 +79,35 @@ sub _check_permissions ( $self, $method_id ) {
     return res [ 403, qq[Insufficient permissions for method "$method_id"] ];
 }
 
-sub api_call ( $self, $method_id, @ ) {
-    my ( $cb, $args );
-
-    # parse $args and $cb
-    if ( is_callback $_[-1] ) {
-        $cb = $_[-1];
-
-        $args = [ @_[ 2 .. $#_ - 1 ] ] if @_ > 3;
-    }
-    else {
-        $args = [ @_[ 2 .. $#_ ] ] if @_ > 2;
-    }
-
-    return $self->api_call_arrayref( $method_id, $args, $cb );
-}
-
-sub api_call_arrayref ( $self, $method_id, $args, $cb = undef ) {
+sub api_call ( $self, $method_id, @args ) {
     my $can_call = $self->api_can_call($method_id);
 
-    if ( !$can_call ) {
-        $cb->($can_call) if defined $cb;
+    return $can_call if !$can_call;
 
-        return $can_call;
+    my $api = $self->{api};
+
+    # get method
+    my $method_cfg = $api->{_method}->{$method_id};
+
+    my $obj = $api->{_obj}->{ $method_cfg->{class_name} };
+
+    my $method_name = $method_cfg->{local_method_name};
+
+    my @res = eval { $obj->$method_name( $self, @args ) };
+
+    # API method eval error
+    if ($@) {
+        $@->sendlog;
+
+        return res 500;
+    }
+
+    # API method didn't returned valid response
+    elsif ( !@res ) {
+        return res 500;
     }
     else {
-        my $api = $self->{api};
-
-        # get method
-        my $method_cfg = $api->{_method}->{$method_id};
-
-        my $obj = $api->{_obj}->{ $method_cfg->{class_name} };
-
-        my $method_name = $method_cfg->{local_method_name};
-
-        if ( defined wantarray ) {
-            my $cv = P->cv;
-
-            # destroy req instance after call
-            {
-                # create API request
-                my $req = bless {
-                    auth => $self,
-                    _cb  => $cv,
-                  },
-                  'Pcore::App::API::Request';
-
-                # call method
-                if ( !eval { $obj->$method_name( $req, $args ? $args->@* : () ); 1 } ) {
-                    $@->sendlog if $@;
-                }
-            }
-
-            my $res = $cv->recv;
-
-            return $cb ? $cb->($res) : $res;
-        }
-        else {
-
-            # create API request
-            my $req = bless {
-                auth => $self,
-                _cb  => $cb,
-              },
-              'Pcore::App::API::Request';
-
-            # call method
-            if ( !eval { $obj->$method_name( $req, $args ? $args->@* : () ); 1 } ) {
-                $@->sendlog if $@;
-            }
-
-            return;
-        }
+        return is_res $res[0] ? $res[0] : res @res;
     }
 }
 
