@@ -1,7 +1,7 @@
 package Pcore::HTTP::Response;
 
 use Pcore -class;
-use Pcore::Lib::Scalar qw[is_plain_coderef is_plain_scalarref];
+use Pcore::Lib::Scalar qw[is_plain_scalarref];
 
 with qw[Pcore::Lib::Result::Role];
 
