@@ -4,7 +4,7 @@ use Pcore -const, -class, -export;
 use Pcore::Core::Patch::AnyEvent::DNSCache;
 use Pcore::Lib::CA;
 use HTTP::Parser::XS qw[];
-use Pcore::Lib::Scalar qw[is_ref is_uri is_plain_scalarref is_plain_arrayref is_plain_coderef is_glob is_plain_hashref];
+use Pcore::Lib::Scalar qw[is_ref is_uri is_plain_scalarref is_plain_arrayref is_glob is_plain_hashref];
 use AnyEvent::Socket qw[];
 use Errno qw[];
 use IO::Socket::SSL qw[$SSL_ERROR SSL_WANT_READ SSL_WANT_WRITE SSL_VERIFY_NONE SSL_VERIFY_PEER];
