@@ -1,7 +1,7 @@
 package Pcore::WebSocket::Handle;
 
 use Pcore -const, -role, -res;
-use Pcore::Lib::Scalar qw[is_ref weaken];
+use Pcore::Lib::Scalar qw[is_ref weaken is_plain_scalarref];
 use Pcore::Lib::Text qw[decode_utf8 encode_utf8];
 use Pcore::Lib::UUID qw[uuid_v4_str];
 use Pcore::Lib::Data qw[to_b64 to_xor];
@@ -20,7 +20,7 @@ use overload    #
 # https://tools.ietf.org/html/rfc7692#page-10
 # https://www.igvita.com/2013/11/27/configuring-and-optimizing-websocket-compression/
 
-requires qw[_on_connect _on_disconnect _on_text _on_binary];
+requires qw[_on_connect _on_disconnect _on_text _on_bin];
 
 has max_message_size => 1_024 * 1_024 * 100;    # PositiveOrZeroInt, 0 - do not check message size
 has compression      => ();                     # Bool, use permessage_deflate compression
@@ -87,30 +87,22 @@ sub accept ( $self, $req, %args ) {    ## no critic qw[Subroutines::ProhibitBuil
 
     my $protocol = ${"$self\::PROTOCOL"};
 
-    $self = $self->new(%args);
-
-    state $on_error = sub ( $self, $req, $status, $reason = undef ) {
-        $req->return_xxx(400);
-
-        $self->_set_status( $status, $reason );
-
-        return $self;
-    };
-
     # websocket version is not specified or not supported
-    return $on_error->( $self, $req, 1002 ) if !$env->{HTTP_SEC_WEBSOCKET_VERSION} || $env->{HTTP_SEC_WEBSOCKET_VERSION} ne $WEBSOCKET_VERSION;
+    return 400 if !$env->{HTTP_SEC_WEBSOCKET_VERSION} || $env->{HTTP_SEC_WEBSOCKET_VERSION} ne $WEBSOCKET_VERSION;
 
     # websocket key is not specified
-    return $on_error->( $self, $req, 1002 ) if !$env->{HTTP_SEC_WEBSOCKET_KEY};
+    return 400 if !$env->{HTTP_SEC_WEBSOCKET_KEY};
 
     # check websocket protocol
     if ($protocol) {
-        return $on_error->( $self, $req, 1002 ) if !$env->{HTTP_SEC_WEBSOCKET_PROTOCOL} || $env->{HTTP_SEC_WEBSOCKET_PROTOCOL} !~ /\b$protocol\b/smi;
+        return 400 if !$env->{HTTP_SEC_WEBSOCKET_PROTOCOL} || $env->{HTTP_SEC_WEBSOCKET_PROTOCOL} !~ /\b$protocol\b/smi;
     }
     elsif ( $env->{HTTP_SEC_WEBSOCKET_PROTOCOL} ) {
-        return $on_error->( $self, $req, 1002 );
+        return 400;
     }
 
+    $self = $self->new(%args);
+
     # server send unmasked frames
     $self->{_send_masked} = 0;
 
@@ -134,14 +126,21 @@ sub accept ( $self, $req, %args ) {    ## no critic qw[Subroutines::ProhibitBuil
         }
     }
 
-    # accept websocket connection
-    my $h = $req->accept_websocket( \@headers );
+    Coro::async_pool {
 
-    # store connestion
-    $SERVER_CONN->{ $self->{id} } = $self;
+        # accept websocket connection
+        my $h = $req->accept_websocket( \@headers );
 
-    # start listen
-    return $self->__on_connect($h);
+        # store connestion
+        $SERVER_CONN->{ $self->{id} } = $self;
+
+        # start listen
+        $self->__on_connect($h);
+
+        return;
+    };
+
+    return;
 }
 
 sub connect ( $self, $uri, %args ) {    ## no critic qw[Subroutines::ProhibitBuiltinHomonyms]
@@ -242,14 +241,22 @@ sub connect ( $self, $uri, %args ) {    ## no critic qw[Subroutines::ProhibitBui
     return $self->__on_connect($h);
 }
 
-sub send_text ( $self, $data_ref ) {
-    $self->{_h}->write( $self->_build_frame( 1, $self->{_compression}, 0, 0, $WEBSOCKET_OP_TEXT, $data_ref ) );
+sub send_text ( $self, $data ) {
+    my $ref = is_plain_scalarref $data ? $data : \$data;
+
+    $ref = \encode_utf8 $ref->$* if utf8::is_utf8 $ref->$*;
+
+    $self->{_h}->write( $self->_build_frame( 1, $self->{_compression}, 0, 0, $WEBSOCKET_OP_TEXT, $ref ) );
 
     return;
 }
 
-sub send_binary ( $self, $data_ref ) {
-    $self->{_h}->write( $self->_build_frame( 1, $self->{_compression}, 0, 0, $WEBSOCKET_OP_BINARY, $data_ref ) );
+sub send_bin ( $self, $data ) {
+    my $ref = is_plain_scalarref $data ? $data : \$data;
+
+    $ref = \encode_utf8 $ref->$* if utf8::is_utf8 $ref->$*;
+
+    $self->{_h}->write( $self->_build_frame( 1, $self->{_compression}, 0, 0, $WEBSOCKET_OP_BINARY, $ref ) );
 
     return;
 }
@@ -586,7 +593,7 @@ sub _on_frame ( $self, $header, $msg, $payload_ref ) {
 
         # BINARY message
         elsif ( $header->{op} == $WEBSOCKET_OP_BINARY ) {
-            $self->_on_binary($payload_ref) if $payload_ref;
+            $self->_on_bin($payload_ref) if $payload_ref;
         }
 
         # CLOSE message
@@ -631,13 +638,13 @@ sub _on_frame ( $self, $header, $msg, $payload_ref ) {
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
 ## |    3 |                      | Subroutines::ProhibitExcessComplexity                                                                          |
-## |      | 147                  | * Subroutine "connect" with high complexity score (24)                                                         |
-## |      | 371                  | * Subroutine "__on_connect" with high complexity score (28)                                                    |
-## |      | 541                  | * Subroutine "_on_frame" with high complexity score (29)                                                       |
+## |      | 146                  | * Subroutine "connect" with high complexity score (24)                                                         |
+## |      | 378                  | * Subroutine "__on_connect" with high complexity score (28)                                                    |
+## |      | 548                  | * Subroutine "_on_frame" with high complexity score (29)                                                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 257, 263, 313        | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
+## |    3 | 264, 270, 320        | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 485, 487, 489        | NamingConventions::ProhibitAmbiguousNames - Ambiguously named variable "second"                                |
+## |    3 | 492, 494, 496        | NamingConventions::ProhibitAmbiguousNames - Ambiguously named variable "second"                                |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
